#!/bin/env python
# coding=utf-8
from __future__ import division, print_function
import msgpack
import numpy as np
import scipy.linalg as la
import math
import sys
import numpy as np
import time
import types
import random
import time
from tools3d.mesh import Mesh
from tools3d.affine import *
from index import *


def estimate(unit_l, unit_a, config):
	n_runs = 5

	# derived num
	npos = int(config['volume']/unit_l**3)
	nori = len(DiscreteOrientation(unit_a))
	ndir = len(DiscreteCameraField(config['fovw'], config['fovh'], unit_a))
	
	t_accum = 0
	for i in range(n_runs):
		index = np.random.rand(ndir,npos,nori,2)
		data = np.random.rand(ndir)
		depth = np.random.rand(480,640)

		t0 = time.time()

		# collect omega
		depth.min()
		depth.max()

		flag = np.ones([npos,nori], bool)
		for (i,o) in enumerate(data):
			flag &= (index[i,:,:,0]<o) & (o<index[i,:,:,1])
		t1 = time.time()

		t_accum += t1-t0

	avg = t_accum/n_runs
	return avg

def list_parameter(config):
	"""
	config requires
	* max_runtime
	* volume
	* fovw
	* fovh

	Returns list of
	(unit length, unit angle, estimated runtime)
	"""
	dur_max = config['max_runtime']
	change_r = 0.95
	unit_l_start = 1.2

	params = []

	# start from coarsest L and A
	unit_l = unit_l_start
	unit_a = math.radians(30)

	# find starting L
	while estimate(unit_l, unit_a, config)<dur_max:
		unit_l *= change_r
	unit_l /= change_r # make it under dur_max

	# make A finer and L coarser to keep it under dur_max
	while unit_l<unit_l_start:
		unit_a *= change_r
		while estimate(unit_l,unit_a,config)>=dur_max:
			unit_l /= change_r
		params.append({
			'lunit': unit_l,
			'aunit': unit_a,
			'runtime': estimate(unit_l,unit_a,config)
			})

	return params


if __name__ == '__main__':
	config = {
		'volume': 67.2,
		'fovw': math.radians(58),
		'fovh': math.radians(45)
	}
	config['max_runtime'] = 0.1

	list_parameter(config)
