#!/bin/env python
# coding=utf-8
"""
This is meant for showing *all* information contained in topological voxel, index(DDF) and result.
As such, we choose 2d visualization over 3d. (because there's cairo)
"""
from __future__ import print_function, division
import numpy as np
import math
import index
import json
from gi.repository import Gtk

class Handler(object):
	def __init__(self, builder):
		self.builder = builder
		self.cx = 0
		self.cy = 0
		self.ct = 0
		self.cp = 0
		self.z = 0
		self.o_tv = None
		self.o_ddf = None

	def onDeleteWindow(self, *args):
		Gtk.main_quit()

	def onExposeXY(self, darea, ctx):
		ctx.set_source_rgb(1,1,1)
		ctx.paint()

		ctx.translate(30,30)

		size = 30

		if self.o_tv != None:
			nx = len(self.o_tv)
			ny = len(self.o_tv[1])

			ctx.set_source_rgb(0,0,0)
			for x in range(nx+1):
				ctx.move_to(0, x*size)
				ctx.rel_line_to(ny*size, 0)

			for y in range(ny+1):
				ctx.move_to(y*size, 0)
				ctx.rel_line_to(0, nx*size)

			ctx.stroke()

			pos_t = {
				0: [],
				1: [(0.5,0.5)],
				2: [(0.25,0.25),(0.75,0.75)]
			}

			def pos_of(x,y,z,i):
				cell = self.o_tv[x][y][z]
				cx = (y+pos_t[len(cell)][i][1])*size
				cy = (x+pos_t[len(cell)][i][0])*size
				return (cx,cy)

			if self.o_tv != None:
				

				for x in range(nx):
					for y in range(ny):
						cell = self.o_tv[x][y][self.z]
						for (ir,region) in enumerate(cell):
							cx, cy = pos_of(x,y,self.z,ir)

							# draw blocked path for Z-
							
							if self.z>=1:
								for ir_down in range(len(self.o_tv[x][y][self.z-1])):
									if ir_down in region['connections'][4]:
										continue

									cxn, cyn = pos_of(x, y, self.z-1, ir_down)
									
									mx = (cx+cxn)*0.5
									my = (cy+cyn)*0.5
									ctx.set_source_rgba(0.8,0.2,0.2,0.5)
									ctx.rectangle(mx-5,my-5,10,10)
									ctx.fill()

									ctx.set_source_rgb(0.6,0.6,0.6)
									ctx.move_to(cx,cy)
									ctx.line_to(mx, my)
									ctx.stroke()

							ctx.set_source_rgb(0.4,0.4,0.4)
							for ir_neighbor in region['connections'][0]:
								cxn, cyn = pos_of(x-1, y, self.z, ir_neighbor)
								ctx.move_to(cx,cy)
								ctx.line_to(cxn, cyn)
								ctx.stroke()

							for ir_neighbor in region['connections'][2]:
								cxn, cyn = pos_of(x, y-1, self.z, ir_neighbor)
								ctx.move_to(cx,cy)
								ctx.line_to(cxn, cyn)
								ctx.stroke()

							ctx.set_source_rgba(0.2,0.2,0.8,0.3)
							if len(cell)==1 and region['empty']:
								ctx.arc(cx, cy, 6, 0, 2*math.pi)
							else:
								ctx.arc(cx, cy, 3, 0, 2*math.pi)
							ctx.fill()

						# cursor
						if (x,y) == (self.cx,self.cy):
							ctx.set_source_rgba(0,0,1,0.2)
							ctx.rectangle(size*y, size*x, size, size)
							ctx.fill()

					#draw_cell()

	def onExposeDDF(self, darea, ctx):
		if self.o_ddf == None:
			return

		ddf_minmax = self.o_ddf.samples_full[self.cx+1, self.cy+1, self.z+1]
		size = darea.get_allocation().width // ddf_minmax.shape[1]

		if darea == self.builder.get_object('drawingarea_ddf_min'):
			ddf = ddf_minmax[:,:,0]
		elif darea == self.builder.get_object('drawingarea_ddf_max'):
			ddf = ddf_minmax[:,:,1]
		else:
			return
		
		# draw cells
		for it in range(ddf.shape[0]):
			for ip in range(ddf.shape[1]):
				v = ddf[it,ip]/10
				ctx.set_source_rgb(v,v,v)
				ctx.rectangle(ip*size, it*size, size, size)
				ctx.fill()

		# draw cursor
		ctx.set_source_rgba(0,0,1,0.8)
		ctx.rectangle(size*self.cp, size*self.ct, size, size)
		ctx.stroke()


	def onButtonPressXY(self, w, ev):
		size = 30
		iy = (ev.x-30)//size
		ix = (ev.y-30)//size
		self.cx = ix
		self.cy = iy
		self.updateLayer()

	def onButtonPressDDF(self, w, ev):
		ddf = self.o_ddf.samples_full[self.cx+1, self.cy+1, self.z+1]
		size = w.get_allocation().width // ddf.shape[1]

		self.cp = ev.x // size
		self.ct = ev.y // size
		self.updateLayer()

	def onPressZUp(self, *args):
		if self.o_tv != None:
			nz = len(self.o_tv[0][0])
		else:
			nz = 1

		self.z = min(nz-1, self.z + 1)
		self.updateLayer()

	def onPressZDown(self, *args):
		self.z = max(0, self.z - 1)
		self.updateLayer()

	def onTVFileSet(self, w):
		print(w.get_filename())
		self.o_tv = json.load(open(w.get_filename(),'r'))

	def onDDFFileSet(self, w):
		print(w.get_filename())
		self.o_ddf = index.Index.load_from(w.get_filename())

	def updateLayer(self):
		self.builder.get_object('label_pos').set_text('(x,y,z)=(%d,%d,%d) (θ,φ)=(%d,%d)'%(self.cx,self.cy,self.z, self.ct, self.cp))
		if self.o_ddf != None:
			ddf_minmax = self.o_ddf.samples_full[self.cx+1, self.cy+1, self.z+1]
			self.builder.get_object('label_dist').set_text('dist=[%.3f,%.3f]'%(ddf_minmax[self.ct,self.cp,0], ddf_minmax[self.ct,self.cp,1]))

		self.builder.get_object('drawingarea_xy').queue_draw()
		self.builder.get_object('drawingarea_ddf_min').queue_draw()
		self.builder.get_object('drawingarea_ddf_max').queue_draw()
		


if __name__ == '__main__':
	builder = Gtk.Builder()
	builder.add_from_file("interact_index.glade")
	builder.connect_signals(Handler(builder))

	# setup file chooser filters
	filt = Gtk.FileFilter()
	filt.set_name('JSON Topological Voxel')
	filt.add_pattern('*.json')
	builder.get_object('filechooserbutton_tv').add_filter(filt)

	filt = Gtk.FileFilter()
	filt.set_name('MessagePack DDF')
	filt.add_pattern('*.mp')
	builder.get_object('filechooserbutton_ddf').add_filter(filt)

	# show
	window = builder.get_object('window_main')
	window.show_all()

	Gtk.main()
