"""
Current index supports "obvious" & "empirical" confusion angles.
While obvious angle can be proven trivially, empirical one is not.

This module test if randomly generated samples doesn't break that emprical premise.
"""
from __future__ import print_function, division
import unittest
import index
import numpy as np
import scipy.linalg as la
import cmath

def angle(mat):
	""" Calculate angle of rotation of given matrix. """
	return cmath.phase(la.eig(mat)[0][0])

class OrientationTestCase(unittest.TestCase):
	def setUp(self):
		self.orientation = index.DiscreteOrientation(0.5)
		self.cf_obvious = self.orientation.calc_obvious_confusion()
		self.cf_empirical = self.orientation.calc_empirical_confusion()

	def test_obvious(self):
		self.assertLessEqual(self.cf_empirical, self.cf_obvious)

	def test_empirical(self):
		for iori in xrange(len(self.orientation)):
			center = self.orientation.realize_as_matrix(iori, [0.5,0.5,0.5])
			inv_center = la.inv(center)
			d = np.random.rand(3)
			a = angle(np.dot(self.orientation.realize_as_matrix(iori, d), inv_center))
			self.assertLessEqual(a, self.cf_empirical)
