#!/bin/env python
# coding=utf-8
from __future__ import division, print_function
import numpy as np
import scipy.linalg as la
import scipy.misc
import math
import random
import msgpack
import msgpackrpc
import time
import sys

from index import *


def visualize(index, path):
	nx,ny,nz = index.pos_shape
	dh = index.direction.ntheta
	dw = index.direction.nphi
	
	out_img = np.empty([ny*dh, nx*dw, 3], np.uint8)
	max_dist = index.samples[index.samples<1e3].max()

	for iz in range(nz):
		for ix in range(nx):
			for iy in range(ny):
				ipos = np.ravel_multi_index((ix,iy,iz), index.pos_shape)

				min_img = index.samples[ipos,:,0].reshape([dh,dw])
				max_img = index.samples[ipos,:,1].reshape([dh,dw])
				avg_img = (min_img+max_img)/2

				out_img[iy*dh:(iy+1)*dh, ix*dw:(ix+1)*dw, 0] = 255*min_img / max_dist
				out_img[iy*dh:(iy+1)*dh, ix*dw:(ix+1)*dw, 1] = 255*max_img / max_dist
				out_img[iy*dh:(iy+1)*dh, ix*dw:(ix+1)*dw, 2] = 255*avg_img / max_dist


		scipy.misc.imsave('%s-%02d.png'%(path,iz), out_img)

if __name__ == '__main__':
	ix_path, out_path = sys.argv[1:]
	index = Index.load_from(ix_path)
	visualize(index, out_path)
